#!/usr/bin/env python
# ******************************************************************************
#  $Id$
#
#  Project:  S-57 OGR Translator
#  Purpose:  Script to translate s57 .csv files into C code "data" statements.
#  Author:   Frank Warmerdam, warmerdam@pobox.com
#
# ******************************************************************************
#  Copyright (c) 2001, Frank Warmerdam
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
# ******************************************************************************

import os
import sys

# -----------------------------------------------------------------------------
# EscapeLine - escape anything C-problematic in a line.
# -----------------------------------------------------------------------------


def EscapeLine(ln):
    return ln.replace('"', '\\"')

# -----------------------------------------------------------------------------
#


if __name__ != '__main__':
    print('This module should only be used as a mainline.')
    sys.exit(1)

if len(sys.argv) < 2:
    directory = os.environ['S57_CSV']
else:
    directory = sys.argv[1]


print('char *gpapszS57Classes[] = {')
classes = open(directory + '/s57objectclasses.csv').readlines()

for line in classes:
    print('"%s",' % EscapeLine(line.strip()))

print('NULL };')

print('char *gpapszS57attributes[] = {')
classes = open(directory + '/s57attributes.csv').readlines()

for line in classes:
    print('"%s",' % EscapeLine(line.strip()))

print('NULL };')



# ******************************************************************************
#  $Id$
#
#  Project:  GDAL ECW Driver
#  Purpose:  Script to lookup ECW (GDT) coordinate systems and translate
#            into OGC WKT for storage in $GDAL_HOME/data/ecw_cs.wkt.
#  Author:   Frank Warmerdam, warmerdam@pobox.com
#
# ******************************************************************************
#  Copyright (c) 2003, Frank Warmerdam
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
# ******************************************************************************

import osr

##############################################################################
# rtod(): radians to degrees.


def r2d(rad):
    return float(rad) / 0.0174532925199433

##############################################################################
# load_dict()


def load_dict(filename):
    lines = open(filename).readlines()

    this_dict = {}
    for line in lines:
        if line[:8] != 'proj_name':
            tokens = [token.strip() for token in line.split(',')]

            this_dict[tokens[0]] = tokens

    return this_dict

##############################################################################
# Mainline


# dir = 'M:/software/ER Viewer 2.0c/GDT_Data/'
directory = '/u/data/ecw/gdt/'

dict_list = ['tranmerc', 'lambert1', 'lamcon2', 'utm', 'albersea', 'mercator',
             'obmerc_b', 'grinten', 'cassini', 'lambazea', 'datum_sp']

dict_dict = {}

for item in dict_list:
    dict_dict[item] = load_dict(directory + item + '.dat')
    # print 'loaded: %s' % item

pfile = open(directory + 'project.dat')
pfile.readline()

for line in pfile.readlines():
    try:
        tokens = line.strip().split(',')
        if len(tokens) < 3:
            continue

        tokens = [token.strip() for token in tokens]

        ident = tokens[0]
        typ = tokens[1]

        lsize = float(tokens[2])
        lsize_str = tokens[2]

        dline = dict_dict[typ][ident]

        srs = osr.SpatialReference()

        # Handle translation of the projection parameters.

        if type != 'utm':
            fn = float(dline[1])
            fe = float(dline[2])

        if type == 'tranmerc':
            srs.SetTM(r2d(dline[5]), r2d(dline[4]), float(dline[3]), fe, fn)

        elif type == 'mercator':
            srs.SetMercator(r2d(dline[5]), r2d(dline[4]), float(dline[3]), fe, fn)

        elif type == 'grinten':
            srs.SetVDG(r2d(dline[3]), fe, fn)

        elif type == 'cassini':
            srs.SetCS(r2d(dline[4]), r2d(dline[3]), fe, fn)

        elif type == 'lambazea':
            srs.SetLAEA(r2d(dline[5]), r2d(dline[4]),
                        fe, fn)

        elif type == 'lambert1':
            srs.SetLCC1SP(r2d(dline[5]), r2d(dline[4]),
                          float(dline[3]), fe, fn)

        elif type == 'lamcon2':
            srs.SetLCC(r2d(dline[7]), r2d(dline[8]),
                       r2d(dline[9]), r2d(dline[6]), fe, fn)

# elif type == 'lambert2':
#   false_en = '+y_0=%.2f +x_0=%.2f' \
#   % (float(dline[12])*lsize, float(dline[13])*lsize)
#     result = '+proj=lcc %s +lat_0=%s +lon_0=%s +lat_1=%s +lat_2=%s' \
#           % (false_en, r2d(dline[3]), r2d(dline[4]),
#           r2d(dline[7]), r2d(dline[8]))

        elif type == 'albersea':
            srs.SetACEA(r2d(dline[3]), r2d(dline[4]),
                        r2d(dline[5]), r2d(dline[6]), fe, fn)

# elif type == 'obmerc_b':
#     result = '+proj=omerc %s +lat_0=%s +lonc=%s +alpha=%s +k=%s' \
#              % (false_en, r2d(dline[5]), r2d(dline[6]), r2d(dline[4]), dline[3])

        elif type == 'utm':
            srs.SetUTM(int(dline[1]), dline[2] != 'S')

        # Handle Units from projects.dat file.
        if srs.IsProjected():
            srs.SetAttrValue('PROJCS', ident)
            if lsize_str == '0.30480061':
                srs.SetLinearUnits('US Foot', float(lsize_str))
            elif lsize_str != '1.0':
                srs.SetLinearUnits('unnamed', float(lsize_str))

        wkt = srs.ExportToWkt()
        if wkt:
            print('%s,%s' % (ident, srs.ExportToWkt()))
        else:
            print('%s,LOCAL_CS["%s - (unsupported)"]' % (ident, ident))

    except KeyError:
        print('%s,LOCAL_CS["%s - (unsupported)"]' % (ident, ident))

    except:
        print('cant translate: ', line)
        raise

# Translate datums to their underlying spheroid information.

pfile = open(directory + 'datum.dat')
pfile.readline()

for line in pfile.readlines():
    tokens = [token.strip() for token in line.strip().split(',')]

    ident = tokens[0]

    sp_name = tokens[2]
    dline = dict_dict['datum_sp'][ident]
    srs = osr.SpatialReference()

    if ident == 'WGS84':
        srs.SetWellKnownGeogCS('WGS84')
    elif ident == 'NAD27':
        srs.SetWellKnownGeogCS('NAD27')
    elif ident == 'NAD83':
        srs.SetWellKnownGeogCS('NAD83')
    else:
        srs.SetGeogCS(tokens[1], ident, sp_name, float(dline[2]), float(dline[4]))

    print('%s,%s' % (ident, srs.ExportToWkt()))



#!/usr/bin/env python
# SPDX-License-Identifier: MIT
# Copyright 2021 Even Rouault

# Analyze git log for ossfuzz vulnerabilities

from subprocess import check_output
out = check_output("git log --no-merges --pretty=oneline".split(' ')).decode('utf-8')
result = list(filter(lambda x: 'chromium' in x, out.splitlines()))

categories = {}

def add_occurence(category_id):
    if category_id in categories:
        categories[category_id] += 1
    else:
        categories[category_id] = 1

for line in result:
    pos = line.find(' ')
    commit_id = line[0:pos]

    # Normalize
    msg = line[pos+1:].lower()
    msg = msg.replace('-', ' ')

    # Fix a few typos
    msg = msg.replace('oveflow', 'overflow')
    msg = msg.replace('overfow', 'overflow')
    msg = msg.replace('divison', 'division')
    msg = msg.replace('divizion', 'division')
    msg = msg.replace('divid', 'division')
    msg = msg.replace('use of initialized', 'uninitialized')
    msg = msg.replace('out of bond', 'out of bound')
    msg = msg.replace('procession', 'processing')
    msg = msg.replace('behaviour', 'behavior')
    msg = msg.replace('perforance', 'performance')

    # print(commit_id, msg)
    if 'overflow' in msg or 'out of bound' in msg or 'out of buffer' in msg or 'stack buffer' in msg:
        if 'stack' in msg:
            if 'read' in msg:
                add_occurence('stack_buffer_overflow_read')
            elif 'read' in msg:
                add_occurence('stack_buffer_overflow_write')
            elif 'buffer' in msg:
                add_occurence('stack_buffer_overflow_unspecified')
            else:
                add_occurence('stack_call_overflow')
        elif 'integer' in msg or 'int32' in msg or 'int64 ' in msg or 'int ' in msg or 'size_t' in msg or 'unsigned' in msg:
            if 'unsigned' in msg or 'size_t' in msg:
                if 'harmless' in msg:
                    add_occurence('integer_overflow_unsigned_harmless')
                else:
                    add_occurence('integer_overflow_unsigned')
            else:
                if 'harmless' in msg:
                    add_occurence('integer_overflow_harmless') # one occurrence due to atoi()
                else:
                    add_occurence('integer_overflow_signed')
        elif 'out of bound' in msg or 'heap' in msg or 'buffer' in msg or 'container overflow' in msg or 'array' in msg or 'read overflow' in msg:
            if 'read ' in msg:
                add_occurence('heap_buffer_overflow_read')
            elif 'write ' in msg:
                add_occurence('heap_buffer_overflow_write')
            else:
                add_occurence('buffer_overflow_unspecified')
        else:
            assert False, (commit_id, msg)
    elif 'unsigned integer underflow' in msg:
        add_occurence('unsigned_integer_underflow')
    elif 'oom' in msg or 'allocate' in msg or (('excessive' in msg or 'too large' in msg or 'too much' or 'huge' in msg or 'big' in msg) and ('memory' in msg or 'ram' in msg or 'allocation' in msg)):
        add_occurence('excessive_memory_use')
    elif 'shift' in msg:
        if 'left' in msg:
            add_occurence('invalid_shift_left')
        elif 'right' in msg:
            add_occurence('invalid_shift_right')
        else:
            add_occurence('invalid_shift_unspecified_dir')
    elif 'division' in msg:
        assert 'zero' in msg
        if 'float' in msg or 'harmless' in msg:
            if 'harmless' in msg:
                add_occurence('division_by_zero_floating_point_harmless')
            else:
                add_occurence('division_by_zero_floating_point_unknown_consequence')
        else:
            add_occurence('division_by_zero_integer')
    elif 'leak' in msg:
        if 'error' in msg or 'corrupted' in msg:
            add_occurence('memory_leak_error_code_path')
        else:
            add_occurence('memory_leak_unspecified')
    elif 'unhandled exception' in msg or 'avoid exception' in msg:
        add_occurence('unhandled_exception')
    elif 'null pointer' in msg or 'null ptr' in msg or 'nullptr' in msg or 'nullpointer' in msg or 'null deref' in msg or 'against null' in msg:
        add_occurence('null_pointer_dereference')
    elif 'reference' in msg:
        add_occurence('invalid_memory_dereference')
    elif 'infinite' in msg and 'loop' in msg:
        add_occurence('infinite_loop')
    elif 'double free' in msg:
        add_occurence('double_free')
    elif 'use after free' in msg:
        add_occurence('use_after_free')
    elif 'uninitialized' in msg:
        add_occurence('uninitialized_variable')
    elif 'assert' in msg:
        add_occurence('assertion')
    elif 'exponential' in msg or 'limit number' in msg or 'speed up' in msg or 'timeout' in msg or 'too long' in msg or \
         (('limit' in msg or 'long' in msg or 'large' in msg or 'slow' in msg) and ('loop' in msg or 'delay' in msg or 'time' in msg or 'processing' in msg)) or \
             ('excessive' in msg and 'time' in msg) or 'performance' in msg or 'big processing time' in msg:
        add_occurence('excessive_processing_time')
    elif 'recursion' in msg or 'recursive' in msg or 'deep call' in msg or 'cyclic call' in msg:
        add_occurence('stack_call_overflow')
    elif 'invalid enum' in msg:
        add_occurence('invalid_enum')
    elif 'cast' in msg:
        add_occurence('invalid_cast')
    elif 'negative size' in msg:
        add_occurence('negative_size_allocation')
    elif 'crash' in msg:
        add_occurence('unspecified_crash')
    elif 'ubsan' in msg or 'unspecified behavior' in msg:
        add_occurence('undefined_behavior_unspecified')
    else:
        add_occurence('other_issue')

ar = []
for k, v in categories.items():
    ar.append((k, v))
ar.sort(key=lambda x: x[1], reverse=True)
total = sum(v for k,v in ar)

print('%-55s %4s %4s' % ('Category', 'Count', 'Pct'))
print('-' * 70)
for k, v in ar:
    print('%-55s %4d  %5.02f %%' % (k, v, 100.0 * v / total))
print('-' * 70)
print('%-55s %4d' % ('Total', total))
